clc
close all
clearvars


% add ROBBO library path
addpath('ROBBO_lib')

%% BOP Problem Setup
% do not comment the parameters of the selected example
n = 30;             % Number of var.
tol = [0.02, 0.07]; % User tolerances expressed as percentage (see "Mode")

%% ROBBO input: struct setup
arg      = n; % Objective functions argument 
xLB      = zeros(n,1);
xUB      = ones(n,1);
A        = [];
b        = [];
Aeq      = [];
beq      = [];
x0       = zeros(n,1);
SolvOptions = [];

% input struct1: Bi-objective opt problem info
BOP = struct('NL_const',@NL_const, 'xLB', xLB,'xUB',xUB, 'A', A, 'b', b,'Aeq',Aeq,'beq',beq, 'x0', x0, 'arg' ,arg);


% input struct2: ROBBO functioning mode
Mode = [];
% Default settings (if Mode is empty) -> Mode = struct('tol','percentage','samp','uniform','est',"central");

% Mode.tol  must be: 'percentage' (tolerances expressed as percentages [0,1] of P.front ranges) or 
%                    'absolute'   (tolerances expressed in absolute terms, not in relative terms) 
%
% Mode.samp must be: 'greedy'     (greedy sampling strategy) or 
%                    'uniform'    (greedy sampling strategy with bisection) or
%                    'bisection'  (greedy sampling strategy with uniform partitioning) 
% Mode.est  must be: 'central'    (central approximation) or
%                    'linear'     (linear approximation)

% Example
% Mode = struct('tol','percentage','samp','bisection','est','linear')

%% Algorithm and exact PF
[report] = ROBBO(@F,BOP,tol,SolvOptions,Mode);

% P.front analytical expression (Plot)
figure(2)
subplot('Position',[0.1, 0.5, 0.8, 0.45])
hold on
f1 = 0:0.001:1;
f2 = 1-(f1).^2;
plot(f1,f2,'--b','LineWidth',1.5,DisplayName="P. front")

filename = fullfile('Animation.gif');
delayTime = 0.8; % Gif frame delay
frame = getframe(2);
im = frame2im(frame);
[imind,cm] = rgb2ind(im,256);
imwrite(imind,cm,filename,'gif','WriteMode','append');
imwrite(imind, cm, filename, 'gif',  'WriteMode', 'append', 'DelayTime', delayTime);

%% Functions and contraints definition (BOP: Bi-objective Oprimization Problem)
% Vector-valued objective functions
function z = F(x, n)
    f1 = x(1);
    g  = 1 + 9/(n-1)*sum(x(2:n)) ;
    h  = 1 - (f1/g)^2;
    f2 = g*h;
    z = [f1,f2]';
end
% Nonlinear constraints
function [c, ceq] = NL_const(~,~)
    c   = [];
    ceq = [];
end